"""Gerrit API Data Models"""

from typing import Any

from pydantic import BaseModel, ConfigDict, Field


class Account(BaseModel):
    """Gerrit Account Info"""

    model_config = ConfigDict(populate_by_name=True)

    account_id: int = Field(alias="_account_id")
    name: str | None = None
    email: str | None = None
    username: str | None = None


class Change(BaseModel):
    """Change Basic Info"""

    model_config = ConfigDict(populate_by_name=True)

    id: str
    project: str
    branch: str
    change_id: str
    subject: str
    status: str
    created: str
    updated: str
    insertions: int = 0
    deletions: int = 0
    number: int = Field(alias="_number")
    owner: Account | None = None
    current_revision: str | None = None
    more_changes: bool | None = Field(default=None, alias="_more_changes")

    @property
    def display_id(self) -> str:
        """Return display ID (number ID)"""
        return str(self.number)


class LabelInfo(BaseModel):
    """Label Info"""

    approved: Account | None = None
    rejected: Account | None = None
    recommended: Account | None = None
    disliked: Account | None = None
    value: int | None = None
    default_value: int | None = None
    values: dict[str, str] | None = None
    all: list[dict[str, Any]] | None = None


class MessageInfo(BaseModel):
    """Message Info"""

    id: str
    author: Account | None = None
    date: str
    message: str
    tag: str | None = None


class FileInfo(BaseModel):
    """File Info"""

    status: str | None = None
    binary: bool | None = None
    old_path: str | None = None
    lines_inserted: int | None = None
    lines_deleted: int | None = None
    size_delta: int | None = None
    size: int | None = None


class ChangeDetail(Change):
    """Change Detail"""

    messages: list[MessageInfo] | None = None
    labels: dict[str, LabelInfo] | None = None
    permitted_labels: dict[str, list[str]] | None = None
    reviewers: dict[str, list[Account]] | None = None
    revisions: dict[str, Any] | None = None


class CommentInfo(BaseModel):
    """Comment Info"""

    id: str | None = None
    patch_set: int | None = None
    path: str | None = None
    side: str | None = None
    line: int | None = None
    range: dict[str, Any] | None = None
    message: str
    updated: str | None = None
    author: Account | None = None
    unresolved: bool | None = None
    in_reply_to: str | None = None



class CommentRange(BaseModel):
    """Comment Range"""

    start_line: int
    start_character: int
    end_line: int
    end_character: int


class CommentInput(BaseModel):
    """Comment Input"""

    path: str | None = None
    line: int | None = None
    range: CommentRange | None = None
    message: str
    side: str = "REVISION"
    in_reply_to: str | None = None
    unresolved: bool | None = None


class ReviewInput(BaseModel):
    """Review Request Input"""

    message: str | None = None
    labels: dict[str, int] | None = None
    comments: dict[str, list[CommentInput]] | None = None
    tag: str | None = None
    notify: str | None = None
    drafts: str | None = None
    ready: bool | None = None
    work_in_progress: bool | None = None


class ReviewResult(BaseModel):
    """Review Response Result"""

    labels: dict[str, int] | None = None
    reviewers: dict[str, Any] | None = None
    ready: bool | None = None



"""Gerrit REST API Client"""

import json
from typing import Any

import httpx

from gerrit_cli.client.models import Change, ChangeDetail, CommentInfo, ReviewInput, ReviewResult
from gerrit_cli.utils.exceptions import ApiError, AuthenticationError, NotFoundError


class GerritClient:
    """Gerrit REST API Client"""

    def __init__(self, base_url: str, username: str, password: str) -> None:
        """Initialize Gerrit Client

        Args:
            base_url: Gerrit Server URL
            username: Username
            password: Password or HTTP Token
        """
        self.base_url = base_url.rstrip("/")
        self.username = username
        self.password = password

        # Create httpx client with Basic Auth
        self.client = httpx.Client(
            auth=(username, password),
            headers={"Content-Type": "application/json; charset=UTF-8"},
            timeout=30.0,
        )

    def __enter__(self) -> "GerritClient":
        """Context manager enter"""
        return self

    def __exit__(self, *args: Any) -> None:
        """Context manager exit, close connection"""
        self.close()

    def close(self) -> None:
        """Close HTTP client"""
        self.client.close()

    def _make_request(
        self, method: str, endpoint: str, params: dict[str, Any] | None = None, data: Any = None
    ) -> Any:
        """Unified request handler

        Args:
            method: HTTP method (GET, POST, PUT, DELETE)
            endpoint: API endpoint (will auto prepend /a/)
            params: URL query params
            data: Request body data

        Returns:
            Parsed JSON response

        Raises:
            AuthenticationError: Authentication failed
            NotFoundError: Resource not found
            ApiError: Other API errors
        """
        # Add /a/ prefix for authentication
        if not endpoint.startswith("/a/"):
            endpoint = f"/a/{endpoint.lstrip('/')}"

        url = f"{self.base_url}{endpoint}"

        try:
            # Send request
            if data is not None:
                json_data = json.dumps(data) if not isinstance(data, str) else data
                response = self.client.request(method, url, params=params, content=json_data)
            else:
                response = self.client.request(method, url, params=params)

            # Handle error status codes
            if response.status_code == 401:
                raise AuthenticationError("Authentication failed, please check username and password")
            elif response.status_code == 404:
                raise NotFoundError(f"Resource not found: {endpoint}")
            elif response.status_code >= 400:
                raise ApiError(
                    f"API request failed: {response.status_code} {response.text}",
                    status_code=response.status_code,
                )

            # Parse response
            return self._parse_response(response)

        except httpx.RequestError as e:
            raise ApiError(f"Network request failed: {e}")

    def _parse_response(self, response: httpx.Response) -> Any:
        """Parse Gerrit API response

        Gerrit API response starts with )]}' prefix to prevent XSSI attacks, need to remove it before parsing

        Args:
            response: HTTP response object

        Returns:
            Parsed JSON data (dict or list)
        """
        text = response.text

        # Remove Gerrit security prefix
        if text.startswith(")]}'\n"):
            text = text[5:]

        # Empty response
        if not text.strip():
            return {}

        try:
            return json.loads(text)
        except json.JSONDecodeError as e:
            raise ApiError(f"JSON parse failed: {e}")

    # ==================== Changes API ====================

    def list_changes(
        self, query: str = "status:open", options: list[str] | None = None, limit: int = 25
    ) -> list[Change]:
        """List changes

        Args:
            query: Query conditions (default: status:open)
            options: Return options list (e.g. CURRENT_REVISION, LABELS etc.)
            limit: Result limit

        Returns:
            List of Change objects
        """
        params: dict[str, Any] = {"q": query, "n": limit}

        # Add return options
        if options:
            for opt in options:
                params["o"] = opt

        data = self._make_request("GET", "/changes/", params=params)

        # Parse as Change object list
        return [Change.model_validate(item) for item in data]

    def get_change(self, change_id: str, options: list[str] | None = None) -> ChangeDetail:
        """Get details of a single change

        Args:
            change_id: Change ID (can be numeric ID, Change-Id or full path)
            options: Return options list

        Returns:
            ChangeDetail object
        """
        params = {}
        if options:
            for opt in options:
                params["o"] = opt

        data = self._make_request("GET", f"/changes/{change_id}", params=params)
        return ChangeDetail.model_validate(data)

    def get_change_comments(self, change_id: str) -> dict[str, list[CommentInfo]]:
        """Get all comments for a change

        Args:
            change_id: Change ID

        Returns:
            Map of file path to comment list
        """
        data = self._make_request("GET", f"/changes/{change_id}/comments")

        # Parse comments
        result: dict[str, list[CommentInfo]] = {}
        for file_path, comments in data.items():
            result[file_path] = [CommentInfo.model_validate(c) for c in comments]

        return result

    def get_change_detail(self, change_id: str) -> ChangeDetail:
        """Get change details (including messages and labels)

        Args:
            change_id: Change ID

        Returns:
            ChangeDetail object
        """
        return self.get_change(
            change_id, options=["CURRENT_REVISION", "MESSAGES", "DETAILED_LABELS", "DETAILED_ACCOUNTS"]
        )

    # ==================== Review API ====================

    def set_review(
        self, change_id: str, revision_id: str, review_input: ReviewInput
    ) -> ReviewResult:
        """Send review (score and comment)

        Args:
            change_id: Change ID
            revision_id: Revision ID (use "current" for current revision)
            review_input: Review input data

        Returns:
            ReviewResult object
        """
        data = review_input.model_dump(exclude_none=True)
        result = self._make_request(
            "POST", f"/changes/{change_id}/revisions/{revision_id}/review", data=data
        )
        return ReviewResult.model_validate(result)

    def add_comment(self, change_id: str, message: str) -> ReviewResult:
        """Simplified add comment interface

        Args:
            change_id: Change ID
            message: Comment content

        Returns:
            ReviewResult object
        """
        review_input = ReviewInput(message=message)
        return self.set_review(change_id, "current", review_input)

    # ==================== Files and Diffs API ====================

    def get_change_files(self, change_id: str, revision_id: str = "current") -> dict[str, Any]:
        """Get list of files changed in a revision

        Args:
            change_id: Change ID
            revision_id: Revision ID (default: "current")

        Returns:
            Dictionary mapping file paths to file info
            Format: {
                "file_path": {
                    "status": "M",  # M=Modified, A=Added, D=Deleted
                    "lines_inserted": 10,
                    "lines_deleted": 5,
                    "size_delta": 100,
                    "size": 1000
                }
            }
        """
        data = self._make_request("GET", f"/changes/{change_id}/revisions/{revision_id}/files/")
        return data

    def get_file_diff(
        self, change_id: str, file_path: str, revision_id: str = "current", context: int = 5
    ) -> dict[str, Any]:
        """Get diff for a specific file

        Args:
            change_id: Change ID
            file_path: File path
            revision_id: Revision ID (default: "current")

        Returns:
            Diff data in Gerrit format
        """
        from urllib.parse import quote

        # URL encode the file path
        encoded_path = quote(file_path, safe="")
        params = {"context": context}
        data = self._make_request(
            "GET",
            f"/changes/{change_id}/revisions/{revision_id}/files/{encoded_path}/diff",
            params=params,
        )
        return data

    def get_all_diffs(
        self, change_id: str, revision_id: str = "current", context: int = 5
    ) -> dict[str, Any]:
        """Get diffs for all files in a change

        Args:
            change_id: Change ID
            revision_id: Revision ID (default: "current")

        Returns:
            Dictionary mapping file paths to their diff data
        """
        files = self.get_change_files(change_id, revision_id)
        diffs = {}

        for file_path in files.keys():
            # Skip commit message and other special files
            if file_path in ["/COMMIT_MSG", "/MERGE_LIST"]:
                continue

            try:
                diffs[file_path] = self.get_file_diff(
                    change_id, file_path, revision_id, context=context
                )
            except (ApiError, NotFoundError):
                # Some files might not have diffs (e.g., binary files)
                continue

        return diffs



"""Helper Functions"""

import subprocess


class GitOperationError(Exception):
    """Git Operation Error"""

    pass


def run_git_command(command: list[str], cwd: str | None = None) -> tuple[bool, str]:
    """Run Git Command

    Args:
        command: List of Git command arguments (e.g. ['git', 'status'])
        cwd: Working directory (optional)

    Returns:
        Tuple of (Success, Output)
    """
    try:
        result = subprocess.run(
            command,
            cwd=cwd,
            capture_output=True,
            text=True,
            check=False,
        )
        return result.returncode == 0, result.stdout.strip()
    except Exception as e:
        return False, str(e)


def check_git_repository(path: str = ".") -> bool:
    """Check if current directory is a Git repository

    Args:
        path: Path to check

    Returns:
        True if it is a Git repository
    """
    success, _ = run_git_command(["git", "rev-parse", "--git-dir"], cwd=path)
    return success


def get_current_branch() -> str | None:
    """Get current Git branch name

    Returns:
        Branch name, or None if not on a branch
    """
    success, output = run_git_command(["git", "rev-parse", "--abbrev-ref", "HEAD"])
    if success and output != "HEAD":
        return output
    return None


def check_working_directory_clean() -> tuple[bool, str]:
    """Check if working directory is clean (no uncommitted changes)

    Returns:
        Tuple of (Is Clean, Status Description)
    """
    success, output = run_git_command(["git", "status", "--porcelain"])
    if not success:
        return False, "Failed to check Git status"

    if not output:
        return True, "Working directory clean"

    # Count changes
    lines = output.strip().split("\n")
    staged = sum(1 for line in lines if line[0] in "MADRC")
    unstaged = sum(1 for line in lines if line[1] in "MD")
    untracked = sum(1 for line in lines if line.startswith("??"))

    status_parts = []
    if staged > 0:
        status_parts.append(f"{staged} staged changes")
    if unstaged > 0:
        status_parts.append(f"{unstaged} unstaged changes")
    if untracked > 0:
        status_parts.append(f"{untracked} untracked files")

    return False, ", ".join(status_parts)


def stash_changes(include_untracked: bool = True) -> tuple[bool, str]:
    """Stash current changes

    Args:
        include_untracked: Whether to include untracked files

    Returns:
        Tuple of (Success, Message)
    """
    command = ["git", "stash", "push", "-m", "gerrit-cli: auto stash before fetch"]
    if include_untracked:
        command.append("--include-untracked")

    success, output = run_git_command(command)
    if success:
        return True, "Stashed current changes"
    return False, f"Stash failed: {output}"


def pop_stash() -> tuple[bool, str]:
    """Pop latest stash

    Returns:
        Tuple of (Success, Message)
    """
    success, output = run_git_command(["git", "stash", "pop"])
    if success:
        return True, "Restored previous changes"
    return False, f"Stash pop failed: {output}"


def fetch_change_ref(change_number: str, ref_spec: str) -> tuple[bool, str]:
    """Fetch change ref from Gerrit

    Args:
        change_number: Change number
        ref_spec: Ref spec (e.g. refs/changes/45/12345/1)

    Returns:
        Tuple of (Success, Message)
    """
    success, output = run_git_command(["git", "fetch", "origin", ref_spec])
    if success:
        return True, f"Fetched change {change_number}"
    return False, f"Fetch failed: {output}"


def checkout_branch(branch_name: str, create: bool = True) -> tuple[bool, str]:
    """Checkout branch

    Args:
        branch_name: Branch name
        create: Whether to create new branch

    Returns:
        Tuple of (Success, Message)
    """
    if create:
        command = ["git", "checkout", "-b", branch_name]
    else:
        command = ["git", "checkout", branch_name]

    success, output = run_git_command(command)
    if success:
        return True, f"Switched to branch {branch_name}"
    return False, f"Failed to checkout branch: {output}"


def checkout_fetch_head() -> tuple[bool, str]:
    """Checkout FETCH_HEAD

    Returns:
        Tuple of (Success, Message)
    """
    success, output = run_git_command(["git", "checkout", "FETCH_HEAD"])
    if success:
        return True, "Switched to FETCH_HEAD"
    return False, f"Checkout failed: {output}"


def branch_exists(branch_name: str) -> bool:
    """Check if branch exists

    Args:
        branch_name: Branch name

    Returns:
        True if branch exists
    """
    success, _ = run_git_command(["git", "rev-parse", "--verify", branch_name])
    return success


def delete_branch(branch_name: str, force: bool = False) -> tuple[bool, str]:
    """Delete branch

    Args:
        branch_name: Branch name
        force: Whether to force delete

    Returns:
        Tuple of (Success, Message)
    """
    flag = "-D" if force else "-d"
    success, output = run_git_command(["git", "branch", flag, branch_name])
    if success:
        return True, f"Deleted branch {branch_name}"
    return False, f"Failed to delete branch: {output}"


def get_git_remote_url(remote_name: str = "origin") -> str | None:
    """Get Git remote URL

    Args:
        remote_name: Remote name (default: origin)

    Returns:
        Remote URL, or None if not found
    """
    success, output = run_git_command(["git", "remote", "get-url", remote_name])
    if success:
        return output.strip()
    return None


def get_repo_root() -> str | None:
    """Get Git repository root directory

    Returns:
        Absolute path to repo root, or None if not in a repo
    """
    success, output = run_git_command(["git", "rev-parse", "--show-toplevel"])
    if success:
        return output.strip()
    return None


def check_remote_exists(remote_name: str = "origin") -> bool:
    """Check if specified remote exists

    Args:
        remote_name: Remote name

    Returns:
        True if remote exists
    """
    success, output = run_git_command(["git", "remote"])
    if success:
        remotes = output.strip().split("\n")
        return remote_name in remotes
    return False


def shorten_path(path: str, max_length: int = 60) -> str:
    """Shorten a file path by truncating or compressing.

    Tiered Strategies:
    1. Middle Truncation: 'first/part/.../filename.kt'
    2. Directory Compression: 'f/p/intermediate/filename.kt'
    3. Filename Only: '.../filename.kt'
    4. Filename Truncation: '...name.kt'
    """
    if len(path) <= max_length:
        return path

    if max_length < 5:
        return f"...{path[-(max_length-3):]}" if max_length > 3 else "..."

    parts = path.split("/")
    first = parts[0]
    last = parts[-1]

    # Tier 1: Middle Truncation "first/.../last" (at least 3 parts)
    if len(parts) >= 3:
        fixed_parts_len = len(first) + 5 + len(last)  # 5 for "/.../"
        if fixed_parts_len <= max_length:
            remaining = max_length - fixed_parts_len
            middle_parts = []
            curr_len = 0
            for p in reversed(parts[1:-1]):
                if curr_len + len(p) + 1 <= remaining:
                    middle_parts.insert(0, p)
                    curr_len += len(p) + 1
                else:
                    break
            if middle_parts:
                return f"{first}/.../{'/'.join(middle_parts)}/{last}"
            else:
                return f"{first}/.../{last}"

    # Tier 2: Directory Compression "f/p/intermediate/last"
    if len(parts) >= 2:
        # Keep last part, compress others to 1 char + /
        compressed_parts = [p[0] for p in parts[:-1]]
        compressed_prefix = "/".join(compressed_parts) + "/"
        if len(compressed_prefix) + len(last) <= max_length:
            return f"{compressed_prefix}{last}"

    # Tier 3: Filename Only ".../last"
    if len(last) + 4 <= max_length:
        return f".../{last}"

    # Tier 4: Filename Truncation "...last"
    return f"...{last[-(max_length-3):]}"



"""Output Formatter Module"""

from gerrit_cli.formatters.base import Formatter
from gerrit_cli.formatters.json import JsonFormatter
from gerrit_cli.formatters.table import TableFormatter


def get_formatter(format_type: str) -> Formatter:
    """Get formatter instance

    Args:
        format_type: Format type (table or json)

    Returns:
        Formatter instance

    Raises:
        ValueError: Unsupported format type
    """
    if format_type == "table":
        return TableFormatter()
    elif format_type == "json":
        return JsonFormatter()
    else:
        raise ValueError(f"Unsupported format type: {format_type}")


__all__ = ["Formatter", "TableFormatter", "JsonFormatter", "get_formatter"]



"""Table Formatter"""

from datetime import datetime

from rich.console import Console
from rich.table import Table
from rich.panel import Panel
from rich.text import Text

from gerrit_cli.client.models import Change, ChangeDetail
from gerrit_cli.formatters.base import Formatter
from gerrit_cli.utils.helpers import shorten_path


class TableFormatter(Formatter):
    """Table formatter using rich"""

    def __init__(self) -> None:
        self.console = Console()

    def format_changes(
        self, changes: list[Change], has_more: bool = False, limit: int | None = None
    ) -> str:
        """Format changes list as a table

        Args:
            changes: List of Change objects

        Returns:
            Formatted table string
        """
        if not changes:
            return "No changes found"

        # Build title
        title = f"Changes ({len(changes)} items)"
        if has_more:
            title += " [yellow](more available)[/yellow]"
        if limit:
            title += f" (limit: {limit})"
 
        table = Table(title=title)
        table.add_column("ID", style="cyan", no_wrap=True)
        table.add_column("Subject", style="white")
        table.add_column("Owner", style="green")
        table.add_column("Project", style="magenta")
        table.add_column("Status", style="yellow")
        table.add_column("+/-", style="blue", no_wrap=True)
        table.add_column("Updated", style="dim")

        for change in changes:
            owner_name = ""
            if change.owner:
                owner_name = change.owner.name or change.owner.username or ""

            # Format insertions/deletions
            changes_text = f"+{change.insertions}/-{change.deletions}"

            # Format updated time
            updated = self._format_time(change.updated)

            table.add_row(
                change.display_id,
                change.subject[:80],  # Limit length
                owner_name,
                change.project,
                change.status,
                changes_text,
                updated,
            )

        # Capture console output
        with self.console.capture() as capture:
            self.console.print(table)

        return capture.get()

    def format_change_detail(self, change: ChangeDetail, show_comments: bool = False) -> str:
        """Format change details

        Args:
            change: ChangeDetail object
            show_comments: Whether to show comments (not implemented yet)

        Returns:
            Formatted detail string
        """
        output_parts = []

        # Title
        title = Text()
        title.append(f"Change {change.display_id}: ", style="bold cyan")
        title.append(change.subject, style="bold white")

        # Basic Info Panel
        info_lines = [
            f"Project: {change.project}",
            f"Branch: {change.branch}",
            f"Status: {change.status}",
            f"Owner: {change.owner.name if change.owner else 'Unknown'}",
            f"Created: {self._format_time(change.created)}",
            f"Updated: {self._format_time(change.updated)}",
            f"Changes: +{change.insertions}/-{change.deletions}",
        ]

        # Label Labels
        if change.labels:
            label_text = []
            for label_name, label_info in change.labels.items():
                value = label_info.value if label_info.value is not None else 0
                label_text.append(f"{label_name}: {value:+d}")
            info_lines.append(f"Labels: {', '.join(label_text)}")

        info_panel = Panel(
            "\n".join(info_lines), title="Basic Info", title_align="left", border_style="cyan"
        )

        # Message History
        messages_text = []
        if change.messages:
            for msg in change.messages[-5:]:  # Show only last 5 messages
                author_name = msg.author.name if msg.author else "Unknown"
                messages_text.append(
                    f"[{self._format_time(msg.date)}] {author_name}:\n  {msg.message[:200]}"
                )

        # Capture Output
        with self.console.capture() as capture:
            self.console.print(title)
            self.console.print(info_panel)

            if messages_text:
                messages_panel = Panel(
                    "\n\n".join(messages_text),
                    title="Recent Messages",
                    title_align="left",
                    border_style="yellow",
                )
                self.console.print(messages_panel)

        return capture.get()

    def format_change_complete(
        self,
        change: ChangeDetail,
        files: dict | None = None,
        diffs: dict | None = None,
        comments: dict | None = None,
        show_parts: dict[str, bool] | None = None,
    ) -> str:
        """格式化 change 的完整视图（类似 tig show）

        Args:
            change: ChangeDetail 对象
            files: 文件列表数据
            diffs: diff 数据
            comments: 评论数据
            show_parts: 要显示的部分，例如 {"metadata": True, "files": True, ...}

        Returns:
            格式化后的字符串
        """
        if show_parts is None:
            # 默认显示元数据、文件、消息（不含 diff）
            show_parts = {
                "metadata": True,
                "files": True,
                "diff": False,
                "messages": True,
                "comments": False,
            }

        parts = []

        # ========== 元数据部分 ==========
        if show_parts.get("metadata", False):
            parts.append(self._render_metadata_panel(change))

        # ========== 文件列表部分 ==========
        if show_parts.get("files", False) and files:
            parts.append(self._render_files_panel(files, change))

        # ========== DIFF 部分 ==========
        if show_parts.get("diff", False) and diffs:
            parts.append(self._render_diffs_panel(diffs))

        # ========== 消息历史部分 ==========
        if show_parts.get("messages", False) and change.messages:
            parts.append(self._render_messages_panel(change.messages))

        # ========== 评论部分 ==========
        if show_parts.get("comments", False) and comments:
            parts.append(self._render_comments_panel(comments))

        # 合并所有部分
        with self.console.capture() as capture:
            for i, part in enumerate(parts):
                self.console.print(part)
                if i < len(parts) - 1:  # 不在最后一个部分后添加空行
                    self.console.print()

        return capture.get()

    def _render_metadata_panel(self, change: ChangeDetail) -> Panel:
        """渲染元数据 Panel"""
        # 标题
        title = Text()
        title.append(f"Change {change.display_id}: ", style="bold cyan")
        title.append(change.subject, style="bold white")
        title.append(f"  [{change.status}]", style="bold yellow")

        # 内容
        content = Text()
        content.append(f"Project: ", style="cyan")
        content.append(f"{change.project}\n")
        content.append(f"Branch:  ", style="cyan")
        content.append(f"{change.branch}\n")
        content.append(f"Owner:   ", style="cyan")
        content.append(f"{change.owner.name if change.owner else 'Unknown'}\n")
        content.append(f"Created: ", style="cyan")
        content.append(f"{self._format_time(change.created)}\n", style="dim")
        content.append(f"Updated: ", style="cyan")
        content.append(f"{self._format_time(change.updated)}\n", style="dim")
        content.append(f"Changes: ", style="cyan")
        content.append(f"+{change.insertions}/-{change.deletions}\n", style="green")

        # Labels
        if change.labels:
            content.append("\nLabels:\n", style="bold cyan")
            for label_name, label_info in change.labels.items():
                value = label_info.value if label_info.value is not None else 0
                color = "green" if value > 0 else "red" if value < 0 else "white"
                content.append(f"  • {label_name}: ", style="white")
                content.append(f"{value:+d}\n", style=color)

        return Panel(content, title=title, title_align="left", border_style="cyan", padding=(1, 2))

    def _render_files_panel(self, files: dict, change: ChangeDetail) -> Panel:
        """渲染文件列表 Panel"""
        file_count = len([f for f in files.keys() if f not in ["/COMMIT_MSG", "/MERGE_LIST"]])
        title = f"FILES CHANGED ({file_count} files, +{change.insertions}/-{change.deletions})"
 
        # Calculate available width for the file path
        # Console width - borders/padding (4) - Status (3) - Changes (15) - spacings
        # Default Console width is 80 if not detectable
        available_width = self.console.width - 26
        if available_width < 20:
            available_width = 20  # Minimum safeguard
 
        # 创建表格
        table = Table(show_header=False, box=None, padding=(0, 1))
        table.add_column("Status", style="yellow", width=3)
        table.add_column("File", style="cyan", no_wrap=True)
        table.add_column("Changes", style="blue", justify="right", width=15)
 
        for file_path, file_info in files.items():
            # 跳过特殊文件
            if file_path in ["/COMMIT_MSG", "/MERGE_LIST"]:
                continue
 
            # 文件状态
            status = file_info.get("status", "M")
            insertions = file_info.get("lines_inserted", 0)
            deletions = file_info.get("lines_deleted", 0)
 
            changes_str = f"+{insertions} -{deletions}"
 
            display_path = shorten_path(file_path, max_length=available_width)
            table.add_row(status, display_path, changes_str)
 
        return Panel(table, title=title, title_align="left", border_style="blue", padding=(0, 1))

    def _render_diffs_panel(self, diffs: dict) -> Panel:
        """渲染 diff Panel"""
        content = Text()

        for file_path, diff_data in diffs.items():
            # 文件头
            content.append(f"\n{'=' * 80}\n", style="dim")
            content.append(f"diff --git a/{file_path} b/{file_path}\n", style="bold white")
            content.append(f"{'=' * 80}\n", style="dim")

            # 转换 Gerrit diff 为 unified diff 格式
            unified_diff = self._convert_gerrit_diff_to_unified(diff_data)

            # 语法高亮显示 diff
            for line in unified_diff.split("\n"):
                if line.startswith("@@"):
                    content.append(line + "\n", style="cyan bold")
                elif line.startswith("+"):
                    content.append(line + "\n", style="green")
                elif line.startswith("-"):
                    content.append(line + "\n", style="red")
                else:
                    content.append(line + "\n", style="white")

        return Panel(content, title="DIFF", title_align="left", border_style="green", padding=(1, 2))

    def _render_messages_panel(self, messages: list) -> Panel:
        """渲染消息历史 Panel"""
        content = Text()

        # 只显示最近 5 条消息
        recent_messages = messages[-5:] if len(messages) > 5 else messages

        for msg in recent_messages:
            author_name = msg.author.name if msg.author else "Unknown"
            time_str = self._format_time(msg.date)

            content.append(f"[{time_str}] ", style="dim")
            content.append(f"{author_name}:\n", style="bold cyan")

            # 处理消息内容（可能有多行）
            for line in msg.message.split("\n"):
                content.append(f"  {line}\n")

            content.append("\n")

        title = f"RECENT MESSAGES ({len(recent_messages)})"
        return Panel(content, title=title, title_align="left", border_style="yellow", padding=(1, 2))

    def _render_comments_panel(self, comments: dict) -> Panel:
        """渲染评论 Panel"""
        content = Text()

        for file_path, comment_list in comments.items():
            if not comment_list:
                continue

            content.append(f"\n{file_path}:\n", style="bold cyan")

            for comment in comment_list:
                author = comment.author.name if comment.author else "Unknown"
                line = comment.line if comment.line else "?"

                content.append(f"  Line {line} - ", style="dim")
                content.append(f"{author}: ", style="bold")
                content.append(f"{comment.message}\n")

        return Panel(
            content, title="INLINE COMMENTS", title_align="left", border_style="magenta", padding=(1, 2)
        )

    def _convert_gerrit_diff_to_unified(self, gerrit_diff: dict) -> str:
        """将 Gerrit diff 格式转换为 unified diff 格式

        Gerrit diff 格式：
        {
            "content": [
                {"ab": ["context line"]},  # 上下文行
                {"a": ["old line"], "b": ["new line"]},  # 修改的行
                ...
            ]
        }
        """
        lines = []

        for section in gerrit_diff.get("content", []):
            if "ab" in section:
                # 上下文行（未修改）
                for line in section["ab"]:
                    lines.append(f" {line}")
            elif "skip" in section:
                # 跳过的行（仅在提供了 context 参数时出现）
                lines.append(f"@@ ... skipped {section['skip']} lines ... @@")
            elif "a" in section and "b" in section:
                # 修改的行（先删除后添加）
                for line in section["a"]:
                    lines.append(f"-{line}")
                for line in section["b"]:
                    lines.append(f"+{line}")
            elif "a" in section:
                # 仅删除的行
                for line in section["a"]:
                    lines.append(f"-{line}")
            elif "b" in section:
                # 仅添加的行
                for line in section["b"]:
                    lines.append(f"+{line}")

        return "\n".join(lines)

    def _format_time(self, time_str: str) -> str:
        """Format time string

        Args:
            time_str: Gerrit time string (Format: 2025-01-01 00:00:00.000000000)

        Returns:
            Formatted time string
        """
        try:
            # Gerrit Time Format: 2025-01-01 00:00:00.000000000
            # Remove nanoseconds
            if "." in time_str:
                time_str = time_str.split(".")[0]

            dt = datetime.strptime(time_str, "%Y-%m-%d %H:%M:%S")

            # Calculate time difference
            now = datetime.now()
            diff = now - dt

            if diff.days > 365:
                return f"{diff.days // 365} years ago"
            elif diff.days > 30:
                return f"{diff.days // 30} months ago"
            elif diff.days > 0:
                return f"{diff.days} days ago"
            elif diff.seconds > 3600:
                return f"{diff.seconds // 3600} hours ago"
            elif diff.seconds > 60:
                return f"{diff.seconds // 60} minutes ago"
            else:
                return "Just now"
        except Exception:
            return time_str



"""gerrit show 命令的部分显示工具"""

# 可用部分及其缩写
AVAILABLE_PARTS = {
    "metadata": "m",
    "files": "f",
    "diff": "d",
    "messages": "msg",
    "comments": "c",
}

# 默认显示的部分（不含 diff，加快速度）
DEFAULT_PARTS = ["metadata", "files", "messages"]


def parse_parts_option(parts_str: str) -> list[str]:
    """解析 --parts 选项

    支持格式：
    - "all" -> 所有部分
    - "m,f,d" -> 缩写
    - "metadata,files,diff" -> 完整名称
    - "m,files,d" -> 混合

    Args:
        parts_str: --parts 选项的值

    Returns:
        部分名称列表（完整名称）

    Raises:
        ValueError: 如果包含未知的部分名称

    Examples:
        >>> parse_parts_option("m,f,d")
        ['metadata', 'files', 'diff']

        >>> parse_parts_option("metadata,diff")
        ['metadata', 'diff']

        >>> parse_parts_option("all")
        ['metadata', 'files', 'diff', 'messages', 'comments']
    """
    # 特殊值：all
    if parts_str == "all":
        return list(AVAILABLE_PARTS.keys())

    # 解析逗号分隔的值
    parts = []
    abbr_to_full = {abbr: full for full, abbr in AVAILABLE_PARTS.items()}

    for item in parts_str.split(","):
        item = item.strip()

        if not item:
            continue

        # 检查是否是完整名称
        if item in AVAILABLE_PARTS:
            parts.append(item)
        # 检查是否是缩写
        elif item in abbr_to_full:
            parts.append(abbr_to_full[item])
        else:
            # 未知的部分
            available = []
            for full, abbr in AVAILABLE_PARTS.items():
                available.append(f"{full}({abbr})")
            raise ValueError(
                f"Unknown part: '{item}'\n"
                f"Available parts: {', '.join(available)}"
            )

    return parts


def get_parts_to_show(parts_option: str | None = None) -> dict[str, bool]:
    """获取要显示的部分

    Args:
        parts_option: --parts 选项的值，None 表示使用默认

    Returns:
        部分名称到是否显示的映射
        例如：{"metadata": True, "files": True, "diff": False, ...}
    """
    if parts_option:
        parts_list = parse_parts_option(parts_option)
    else:
        parts_list = DEFAULT_PARTS

    # 转换为字典（所有部分默认 False，指定的设为 True）
    return {part: (part in parts_list) for part in AVAILABLE_PARTS.keys()}



"""Change Command Group"""

import sys

import click

from gerrit_cli.client.api import GerritClient
from gerrit_cli.formatters import get_formatter
from gerrit_cli.utils.exceptions import GerritCliError
from gerrit_cli.utils.helpers import (
    branch_exists,
    check_git_repository,
    check_remote_exists,
    check_working_directory_clean,
    checkout_branch,
    checkout_fetch_head,
    delete_branch,
    fetch_change_ref,
    get_current_branch,
    get_git_remote_url,
    get_repo_root,
    stash_changes,
)


@click.group()
def change() -> None:
    """Change related commands"""
    pass


@change.command()
@click.option("-q", "--query", default="status:open", help="Query conditions (default: status:open)")
@click.option("-n", "--limit", default=25, type=int, help="Limit number of results (default: 25)")
@click.option("-o", "--owner", help="Filter by owner (use 'me' for current user)")
@click.option("-p", "--project", help="Filter by project")
@click.option(
    "--format",
    "output_format",
    type=click.Choice(["table", "json"]),
    default="table",
    help="Output format",
)
@click.pass_context
def list(
    ctx: click.Context,
    query: str,
    limit: int,
    owner: str | None,
    project: str | None,
    output_format: str,
) -> None:
    """List changes

    Examples:
        gerrit list
        gerrit list -q "status:merged"
        gerrit list --owner me
        gerrit list --project myproject --format json
    """
    config = ctx.obj["config"]

    try:
        # Build query conditions
        query_parts = [query]

        if owner:
            if owner == "me":
                query_parts.append(f"owner:{config.username}")
            else:
                query_parts.append(f"owner:{owner}")

        if project:
            query_parts.append(f"project:{project}")

        final_query = " ".join(query_parts)

        # Call API
        with GerritClient(config.url, config.username, config.password) as client:
            changes = client.list_changes(
                query=final_query, options=["CURRENT_REVISION", "LABELS", "DETAILED_ACCOUNTS"], limit=limit
            )

        # Check if more available (Gerrit sets _more_changes on the last item)
        has_more = False
        if changes and changes[-1].more_changes:
            has_more = True
 
        # Format output
        formatter = get_formatter(output_format)
        output = formatter.format_changes(changes, has_more=has_more, limit=limit)
        click.echo(output)

    except GerritCliError as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


@change.command()
@click.argument("change_id")
@click.option(
    "--format",
    "output_format",
    type=click.Choice(["table", "json"]),
    default="table",
    help="Output format",
)
@click.option(
    "--parts",
    help=(
        "指定显示的部分（逗号分隔）。"
        "可用: metadata(m), files(f), diff(d), messages(msg), comments(c), all。"
        "默认: m,f,msg"
    ),
)
@click.option(
    "--context",
    type=int,
    default=5,
    help="显示 diff 时，改动部分的上下文行数（默认: 5）。设为较大值可显示更多上下文。",
)
@click.pass_context
def view(
    ctx: click.Context,
    change_id: str,
    output_format: str,
    parts: str | None,
    context: int,
) -> None:
    """查看 change 的详细信息

    默认显示：元数据 + 文件列表 + 消息历史（不含 diff，更快）

    CHANGE_ID 可以是数字 ID 或 Change-Id

    示例：
        # 默认显示（元数据 + 文件 + 消息）
        gerrit show 12345

        # 包含 diff（默认显示改动上下 5 行）
        gerrit show 12345 --parts m,f,d

        # 包含 diff，只显示改动上下 3 行
        gerrit show 12345 --parts diff --context 3

        # 只显示代码差异
        gerrit show 12345 --parts diff
        gerrit show 12345 --parts d

        # 显示所有
        gerrit show 12345 --parts all

        # 自定义组合
        gerrit show 12345 --parts metadata,diff,comments

    可用部分：
        metadata (m)   - 基本信息、Owner、Status、Labels
        files (f)      - 文件列表及统计信息
        diff (d)       - 代码差异
        messages (msg) - 消息历史
        comments (c)   - 内联评论
        all            - 所有部分
    """
    from gerrit_cli.utils.show_parts import get_parts_to_show

    config = ctx.obj["config"]

    try:
        # 解析要显示的部分
        show_parts = get_parts_to_show(parts)

        with GerritClient(config.url, config.username, config.password) as client:
            # 1. 获取基本信息（总是需要）
            change = client.get_change_detail(change_id)

            # 2. 根据需要获取额外数据
            files_data = None
            diffs_data = None
            comments_data = None

            # 获取文件列表（如果需要显示文件或 diff）
            if show_parts["files"] or show_parts["diff"]:
                files_data = client.get_change_files(change_id)

            # 获取 diff（如果需要）
            if show_parts["diff"]:
                # 检查 diff 大小，避免获取超大 diff
                total_lines = change.insertions + change.deletions
                max_lines = 1000  # 最大行数限制

                if total_lines > max_lines:
                    click.echo(
                        f"警告：diff 太大（{total_lines} 行），跳过显示。", err=True
                    )
                    click.echo(
                        f"提示：使用 'gerrit checkout {change_id}' 在本地查看。", err=True
                    )
                    show_parts["diff"] = False
                else:
                    diffs_data = client.get_all_diffs(change_id, context=context)

            # 获取评论（如果需要）
            if show_parts["comments"]:
                comments_data = client.get_change_comments(change_id)

            # 3. 格式化输出
            if output_format == "json":
                import json

                output_data = {
                    "change": change.model_dump()
                    if hasattr(change, "model_dump")
                    else change.__dict__,
                }
                if files_data:
                    output_data["files"] = files_data
                if diffs_data:
                    output_data["diffs"] = diffs_data
                if comments_data:
                    output_data["comments"] = comments_data

                click.echo(json.dumps(output_data, indent=2, ensure_ascii=False))
            else:
                # Table 格式 - 使用完整视图
                formatter = get_formatter(output_format)
                output = formatter.format_change_complete(
                    change,
                    files=files_data,
                    diffs=diffs_data,
                    comments=comments_data,
                    show_parts=show_parts,
                )
                click.echo(output)

    except ValueError as e:
        # --parts 选项解析错误
        click.echo(f"错误: {e}", err=True)
        sys.exit(1)

    except GerritCliError as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


@change.command()
@click.argument("change_id")
@click.option("-m", "--message", help="Comment message")
@click.option("-f", "--file", "file_path", type=click.Path(exists=True), help="Read comment from file")
@click.option("--draft", is_flag=True, help="Save as draft")
@click.pass_context
def comment(
    ctx: click.Context,
    change_id: str,
    message: str | None,
    file_path: str | None,
    draft: bool,
) -> None:
    """Add comment to change

    Examples:
        gerrit change comment 12345 -m "LGTM"
        gerrit change comment 12345 -f comment.txt
    """
    config = ctx.obj["config"]

    try:
        # Get comment content
        if message:
            comment_text = message
        elif file_path:
            with open(file_path, "r", encoding="utf-8") as f:
                comment_text = f.read()
        else:
            click.echo("Error: Must provide -m or -f option", err=True)
            sys.exit(1)

        # Call API
        with GerritClient(config.url, config.username, config.password) as client:
            if draft:
                # TODO: Implement draft comment
                click.echo("Draft feature not implemented yet", err=True)
                sys.exit(1)
            else:
                result = client.add_comment(change_id, comment_text)
                click.echo(f"✓ Comment added to Change {change_id}", err=False)

    except GerritCliError as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


@change.command()
@click.argument("change_id")
@click.option("-b", "--branch", help="Local branch name (default: review/change-{change_id})")
@click.option("--force", is_flag=True, help="Force delete and recreate branch if exists")
@click.option("--no-checkout", is_flag=True, help="Only fetch, do not checkout")
@click.option(
    "--stash/--no-stash",
    default=None,
    help="Auto stash uncommitted changes (will prompt by default)",
)
@click.pass_context
def checkout(
    ctx: click.Context,
    change_id: str,
    branch: str | None,
    force: bool,
    no_checkout: bool,
    stash: bool | None,
) -> None:
    """Checkout change to local branch

    This command fetches a specified change from Gerrit to local, and creates a new branch.
    If the working directory has uncommitted changes, it will prompt you how to handle them.

    CHANGE_ID can be a numeric ID, Change-Id, or full path

    Examples:
        # Checkout change 12345 to new branch
        gerrit checkout 12345

        # Specify branch name
        gerrit checkout 12345 -b my-review-branch

        # Force recreate branch if exists
        gerrit checkout 12345 --force

        # Only fetch without checkout
        gerrit checkout 12345 --no-checkout

        # Auto stash uncommitted changes
        gerrit checkout 12345 --stash
    """
    config = ctx.obj["config"]

    try:
        # 1. Check if inside a Git repository
        if not check_git_repository():
            click.echo("Error: Current directory is not a Git repository", err=True)
            click.echo()
            click.echo("Please cd into the Gerrit project's Git repository before running this command.", err=True)
            click.echo()
            click.echo("Example:")
            click.echo("  cd /path/to/your/gerrit/project")
            click.echo("  gerrit checkout 12345")
            sys.exit(1)

        # 2. Get change info from Gerrit
        click.echo(f"Fetching info for Change {change_id}...")
        with GerritClient(config.url, config.username, config.password) as client:
            change = client.get_change(change_id, options=["CURRENT_REVISION", "DOWNLOAD_COMMANDS", "DETAILED_ACCOUNTS"])

        # 3. Verify current repo matches project
        repo_root = get_repo_root()
        if repo_root:
            click.echo(f"Current repo: {repo_root}")

        # Check if remote exists
        if not check_remote_exists("origin"):
            click.echo()
            click.echo("Warning: 'origin' remote not found in current Git repository", err=True)
            click.echo("Fetch operation may fail.", err=True)
            click.echo()

            if not click.confirm("Continue?", default=False):
                click.echo("Operation cancelled")
                sys.exit(0)
        else:
            # Get remote URL and warn user
            remote_url = get_git_remote_url("origin")
            if remote_url:
                click.echo(f"Remote URL: {remote_url}")

                # Simple check: if remote URL contains project name
                # Note: This is an heuristic check, not 100% reliable
                if change.project not in remote_url:
                    click.echo()
                    click.echo(f"⚠️  Warning: Current repo remote URL does not seem to match Change project", err=True)
                    click.echo(f"   Change Project: {change.project}", err=True)
                    click.echo(f"   Remote URL: {remote_url}", err=True)
                    click.echo()
                    click.echo("Possible reasons:")
                    click.echo("  1. You are running this command in the wrong Git repository")
                    click.echo(f"  2. You need to cd into '{change.project}' project directory")
                    click.echo()

                    if not click.confirm("Still want to continue fetch?", default=False):
                        click.echo("Operation cancelled")
                        click.echo()
                        click.echo("Suggestion:")
                        click.echo(f"  1. Find the Git repo directory for {change.project}")
                        click.echo("  2. cd into that directory")
                        click.echo(f"  3. Run again: gerrit checkout {change_id}")
                        sys.exit(0)

        # 4. Verify change info
        if not change.current_revision:
            click.echo("Error: Cannot get current revision for change", err=True)
            sys.exit(1)

        click.echo()

        # 5. Get ref spec
        # Gerrit ref format: refs/changes/[last 2 digits]/[change number]/[patch set number]
        change_number = change.number
        last_two = change_number % 100
        ref_spec = f"refs/changes/{last_two:02d}/{change_number}/1"

        click.echo(f"Change: {change.subject}")
        click.echo(f"Project: {change.project}")
        click.echo(f"Branch: {change.branch}")
        click.echo(f"Owner: {change.owner.name if change.owner else 'Unknown'}")
        click.echo(f"Ref: {ref_spec}")
        click.echo()

        # 6. Determine branch name
        if branch is None:
            branch_name = f"review/change-{change_number}"
        else:
            branch_name = branch

        # 7. Check if branch exists
        if branch_exists(branch_name):
            if force:
                click.echo(f"Warning: Branch '{branch_name}' exists, deleting...")
                current = get_current_branch()
                if current == branch_name:
                    click.echo("Error: Cannot delete current branch, please switch to another branch first", err=True)
                    sys.exit(1)
                success, msg = delete_branch(branch_name, force=True)
                if not success:
                    click.echo(f"Error: {msg}", err=True)
                    sys.exit(1)
            else:
                click.echo(f"Error: Branch '{branch_name}' already exists", err=True)
                click.echo("Use --force to force delete and recreate, or use -b to specify another branch name")
                sys.exit(1)

        # 8. Check working directory status
        if not no_checkout:
            is_clean, status_msg = check_working_directory_clean()

            if not is_clean:
                click.echo(f"Warning: Uncommitted changes in working directory: {status_msg}")
                click.echo()

                # Decide how to handle based on args
                should_stash = stash
                if should_stash is None:
                    # Ask user
                    choice = click.prompt(
                        "How to handle uncommitted changes?\n"
                        "  1) Stash changes (Recommended)\n"
                        "  2) Cancel operation\n"
                        "  3) Force continue (Changes may be lost)\n"
                        "Please choose",
                        type=click.Choice(["1", "2", "3"]),
                        default="1",
                    )

                    if choice == "1":
                        should_stash = True
                    elif choice == "2":
                        click.echo("Operation cancelled")
                        sys.exit(0)
                    else:  # choice == "3"
                        should_stash = False

                # Perform stash
                if should_stash:
                    click.echo("Stashing current changes...")
                    success, msg = stash_changes(include_untracked=True)
                    if not success:
                        click.echo(f"Error: {msg}", err=True)
                        sys.exit(1)
                    click.echo(f"✓ {msg}")
                    click.echo("  Use 'git stash pop' to restore changes")
                    click.echo()

        # 9. Fetch change
        click.echo(f"Fetching change {change_number} from Gerrit...")
        success, msg = fetch_change_ref(str(change_number), ref_spec)
        if not success:
            click.echo(f"Error: {msg}", err=True)
            sys.exit(1)
        click.echo(f"✓ {msg}")

        # 10. Create and checkout branch
        if not no_checkout:
            click.echo(f"Creating and switching to branch '{branch_name}'...")

            # Checkout FETCH_HEAD first
            success, msg = checkout_fetch_head()
            if not success:
                click.echo(f"Error: {msg}", err=True)
                sys.exit(1)

            # Create new branch
            success, msg = checkout_branch(branch_name, create=True)
            if not success:
                click.echo(f"Error: {msg}", err=True)
                sys.exit(1)

            click.echo(f"✓ {msg}")
            click.echo()
            click.echo(f"Success! Fetched Change {change_number} to branch '{branch_name}'")
            click.echo(f"Current branch: {branch_name}")
        else:
            click.echo()
            click.echo(f"Success! Fetched Change {change_number}")
            click.echo("Use 'git checkout FETCH_HEAD' to check it out")

        # 11. Next steps
        click.echo()
        click.echo("Next steps:")
        click.echo(f"  - View code: git log -1 --stat")
        click.echo(f"  - View diff: git diff {change.branch}")
        if not no_checkout:
            current = get_current_branch()
            if current:
                click.echo(f"  - Return to original branch: git checkout {change.branch}")
            click.echo(f"  - Delete this branch: git checkout {change.branch} && git branch -D {branch_name}")

    except GerritCliError as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)



"""Custom exceptions"""


class GerritCliError(Exception):
    """Base CLI exception"""

    pass


class ConfigError(GerritCliError):
    """Configuration error"""

    pass


class ApiError(GerritCliError):
    """API request error"""

    def __init__(self, message: str, status_code: int | None = None) -> None:
        self.status_code = status_code
        super().__init__(message)


class AuthenticationError(ApiError):
    """Authentication error"""

    pass


class NotFoundError(ApiError):
    """Resource not found"""

    pass



"""Formatter Base Class"""

from abc import ABC, abstractmethod

from gerrit_cli.client.models import Change, ChangeDetail


class Formatter(ABC):
    """Abstract base class for formatters"""

    @abstractmethod
    def format_changes(
        self, changes: list[Change], has_more: bool = False, limit: int | None = None
    ) -> str:
        """Format changes list

        Args:
            changes: List of Change objects

        Returns:
            Formatted string
        """
        pass

    @abstractmethod
    def format_change_detail(self, change: ChangeDetail, show_comments: bool = False) -> str:
        """Format change details

        Args:
            change: ChangeDetail object
            show_comments: Whether to show comments

        Returns:
            Formatted string
        """
        pass



"""Configuration management module"""

import os
from dataclasses import dataclass

from dotenv import load_dotenv

from gerrit_cli.utils.exceptions import ConfigError


@dataclass
class GerritConfig:
    """Gerrit Configuration"""

    url: str
    username: str
    password: str

    @classmethod
    def from_env(cls) -> "GerritConfig":
        """Load configuration from environment variables

        Environment Variables:
            GERRIT_URL: Gerrit Server URL
            GERRIT_USERNAME: Username
            GERRIT_PASSWORD: Password (Preferred)
            GERRIT_TOKEN: HTTP Token (Alternative)

        Returns:
            GerritConfig: Configuration object

        Raises:
            ConfigError: Missing or invalid configuration
        """
        # Load .env file
        load_dotenv()

        url = os.getenv("GERRIT_URL")
        username = os.getenv("GERRIT_USERNAME")
        password = os.getenv("GERRIT_PASSWORD") or os.getenv("GERRIT_TOKEN")

        # Validate required config
        if not all([url, username, password]):
            missing = []
            if not url:
                missing.append("GERRIT_URL")
            if not username:
                missing.append("GERRIT_USERNAME")
            if not password:
                missing.append("GERRIT_PASSWORD or GERRIT_TOKEN")

            raise ConfigError(
                f"Missing required environment variables: {', '.join(missing)}\n"
                f"Please set environment variables or create a .env file (see .env.example)"
            )

        config = cls(url=url, username=username, password=password)
        config.validate()
        return config

    def validate(self) -> None:
        """Validate configuration

        Raises:
            ConfigError: Invalid configuration
        """
        if not self.url.startswith(("http://", "https://")):
            raise ConfigError(f"GERRIT_URL must be a valid HTTP(S) URL, current value: {self.url}")

        # Remove trailing slash
        if self.url.endswith("/"):
            self.url = self.url.rstrip("/")



"""CLI Main Entry Point"""

import sys

import click

from gerrit_cli import __version__
from gerrit_cli.config import GerritConfig
from gerrit_cli.utils.exceptions import GerritCliError


@click.group()
@click.version_option(version=__version__)
@click.pass_context
def main(ctx: click.Context) -> None:
    """Gerrit CLI - Gerrit Code Review Command Line Tool

    Please configure environment variables before use:
    - GERRIT_URL: Gerrit Server URL
    - GERRIT_USERNAME: Username
    - GERRIT_PASSWORD or GERRIT_TOKEN: Password or HTTP Token
    """
    # Ensure ctx.obj exists
    ctx.ensure_object(dict)

    # If viewing help, skip config loading
    if "--help" in sys.argv or len(sys.argv) == 1:
        return

    try:
        # Load config and store in context
        ctx.obj["config"] = GerritConfig.from_env()
    except GerritCliError as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


# Register command groups
from gerrit_cli.commands.change import change
from gerrit_cli.commands.review import review

main.add_command(change)
main.add_command(review)

# Root-level Alias (Scheme B)
main.add_command(change.commands["list"], name="list")
main.add_command(change.commands["view"], name="show")
main.add_command(change.commands["checkout"], name="checkout")

# Group Alias (Scheme A)
main.add_command(change, name="c")


if __name__ == "__main__":
    main()



"""Review Command"""

import re
import sys

import click

from gerrit_cli.client.api import GerritClient
from gerrit_cli.client.models import CommentInput, CommentRange, ReviewInput
from gerrit_cli.utils.exceptions import GerritCliError


@click.command()
@click.argument("change_id")
@click.option("-m", "--message", help="Review message")
@click.option(
    "--code-review",
    type=click.Choice(["+2", "+1", "0", "-1", "-2"]),
    help="Code-Review score",
)
@click.option(
    "--verified",
    type=click.Choice(["+1", "0", "-1"]),
    help="Verified score",
)
@click.option("-f", "--file", "file_path", type=click.Path(exists=True), help="Read message from file")
@click.option(
    "--inline-comment",
    nargs=2,
    type=(str, str),
    multiple=True,
    help="Inline comment: file#location message (location: line, start-end, LxxCxx-LxxCxx)",
)
@click.option("--submit", is_flag=True, help="Submit change after review")
@click.pass_context
def review(
    ctx: click.Context,
    change_id: str,
    message: str | None,
    code_review: str | None,
    verified: str | None,
    file_path: str | None,
    inline_comment: list[tuple[str, str]],
    submit: bool,
) -> None:
    """Send review (score + comment)

    Examples:
        gerrit review 12345 --code-review +2 -m "LGTM"
        gerrit review 12345 --code-review -1 -m "Needs improvement"
        gerrit review 12345 --code-review +2 --verified +1 --submit
        gerrit review 12345 --code-review +2 --verified +1 --submit
        gerrit review 12345 -f review.txt --code-review +1
        gerrit review 12345 --inline-comment src/main.py 10 "Fix typo"
        gerrit review 12345 --inline-comment src/main.py#10 "Fix typo"
        gerrit review 12345 --inline-comment src/main.py#10-20 "Refactor this block"
        gerrit review 12345 --inline-comment src/main.py#L12C13-L12C19 "Specific syntax error"
    """
    config = ctx.obj["config"]

    try:
        # Get message content
        review_message = None
        if message:
            review_message = message
        elif file_path:
            with open(file_path, "r", encoding="utf-8") as f:
                review_message = f.read()

        # Build labels
        labels: dict[str, int] = {}
        if code_review:
            labels["Code-Review"] = int(code_review)
        if verified:
            labels["Verified"] = int(verified)

        # Validation: Must provide message, labels, or inline comments
        if not review_message and not labels and not inline_comment:
            click.echo(
                "Error: Must provide message (-m or -f), scores (--code-review or --verified), or inline comments (--inline-comment)",
                err=True,
            )
            sys.exit(1)

        # Build ReviewInput
        comments: dict[str, list[CommentInput]] = {}
        for location_spec, m in inline_comment:
            if "#" not in location_spec:
                click.echo(f"Error: Invalid inline comment format '{location_spec}'. Expected 'file#location'", err=True)
                sys.exit(1)

            f, l_raw = location_spec.rsplit("#", 1)

            if f not in comments:
                comments[f] = []

            # Parse line or range
            line: int
            comment_range: CommentRange | None = None

            # Check for character range: L12C13-L12C19 (case insensitive)
            char_range_match = re.match(r"^L(\d+)C(\d+)-L(\d+)C(\d+)$", l_raw, re.IGNORECASE)
            if char_range_match:
                start_line, start_char, end_line, end_char = map(int, char_range_match.groups())
                line = end_line
                comment_range = CommentRange(
                    start_line=start_line,
                    start_character=start_char,
                    end_line=end_line,
                    end_character=end_char,
                )
            elif "-" in l_raw:
                try:
                    start, end = map(int, l_raw.split("-"))
                    line = end
                    comment_range = CommentRange(
                        start_line=start,
                        start_character=0,
                        end_line=end,
                        end_character=10000,
                    )
                except ValueError:
                    click.echo(
                        f"Error: Invalid line format '{l_raw}', expected 'line', 'start-end', or 'LnCm-LnCm'",
                        err=True,
                    )
                    sys.exit(1)
            else:
                try:
                    line = int(l_raw)
                except ValueError:
                    click.echo(f"Error: Invalid line number '{l_raw}'", err=True)
                    sys.exit(1)

            comments[f].append(CommentInput(line=line, message=m, range=comment_range))

        review_input = ReviewInput(
            message=review_message,
            labels=labels if labels else None,
            comments=comments if comments else None,
        )

        # Call API
        with GerritClient(config.url, config.username, config.password) as client:
            result = client.set_review(change_id, "current", review_input)

            # Output result
            click.echo(f"✓ Review sent to Change {change_id}")

            if result.labels:
                label_text = ", ".join([f"{k}: {v:+d}" for k, v in result.labels.items()])
                click.echo(f"  Labels: {label_text}")

            if review_message:
                click.echo(f"  Message: {review_message[:100]}{'...' if len(review_message) > 100 else ''}")

            # TODO: Implement submit feature
            if submit:
                click.echo(f"  Note: --submit feature not implemented yet", err=True)

    except GerritCliError as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)



"""Gerrit API Client Module"""

from gerrit_cli.client.models import (
    Account,
    Change,
    ChangeDetail,
    CommentInfo,
    CommentInput,
    ReviewInput,
    ReviewResult,
)

__all__ = [
    "Account",
    "Change",
    "ChangeDetail",
    "CommentInfo",
    "CommentInput",
    "ReviewInput",
    "ReviewResult",
]



"""JSON Formatter"""

import json

from gerrit_cli.client.models import Change, ChangeDetail
from gerrit_cli.formatters.base import Formatter


class JsonFormatter(Formatter):
    """JSON formatter"""

    def format_changes(
        self, changes: list[Change], has_more: bool = False, limit: int | None = None
    ) -> str:
        """Format changes list as JSON

        Args:
            changes: List of Change objects

        Returns:
            JSON string
        """
        data = [change.model_dump(mode="json") for change in changes]
        return json.dumps(data, indent=2, ensure_ascii=False)

    def format_change_detail(self, change: ChangeDetail, show_comments: bool = False) -> str:
        """Format change detail as JSON

        Args:
            change: ChangeDetail object
            show_comments: Whether to show comments (JSON always includes all data)

        Returns:
            JSON string
        """
        data = change.model_dump(mode="json")
        return json.dumps(data, indent=2, ensure_ascii=False)



import pytest
import respx
from httpx import Response
from click.testing import CliRunner
from gerrit_cli.cli import main as cli
from gerrit_cli.client.models import ReviewInput

@pytest.fixture
def runner():
    return CliRunner()

@respx.mock
def test_review_inline_comment(runner):
    # Mock API
    route = respx.post("https://gerrit.example.com/a/changes/12345/revisions/current/review").mock(
        return_value=Response(200, json={"labels": {"Code-Review": 2}})
    )

    result = runner.invoke(
        cli,
        [
            "review",
            "12345",
            "--inline-comment",
            "src/main.py#10",
            "Fix typo",
            "--inline-comment",
            "src/utils.py#20",
            "Refactor this",
            "--inline-comment",
            "src/range.py#10-20",
            "Multi-line comment",
            "--inline-comment",
            "src/utils.py#L12C13-L12C19",
            "Char range",
        ],
        env={
            "GERRIT_URL": "https://gerrit.example.com",
            "GERRIT_USERNAME": "user",
            "GERRIT_PASSWORD": "password",
        },
    )

    assert result.exit_code == 0
    assert "Review sent to Change 12345" in result.output

    # Verify request payload
    assert route.called
    request_data = ReviewInput.model_validate_json(route.calls.last.request.content)
    
    assert request_data.comments is not None
    assert "src/main.py" in request_data.comments
    assert "src/utils.py" in request_data.comments
    assert "src/range.py" in request_data.comments
    
    assert request_data.comments["src/main.py"][0].line == 10
    assert request_data.comments["src/main.py"][0].message == "Fix typo"
    
    assert request_data.comments["src/utils.py"][0].line == 20
    assert request_data.comments["src/utils.py"][0].message == "Refactor this"

    range_comment = request_data.comments["src/range.py"][0]
    assert range_comment.line == 20
    assert range_comment.message == "Multi-line comment"
    assert range_comment.range is not None
    assert range_comment.range.start_line == 10
    assert range_comment.range.start_character == 0
    assert range_comment.range.end_line == 20
    assert range_comment.range.end_character == 10000

    char_comment = request_data.comments["src/utils.py"][1]
    assert char_comment.line == 12
    assert char_comment.message == "Char range"
    assert char_comment.range is not None
    assert char_comment.range.start_line == 12
    assert char_comment.range.start_character == 13
    assert char_comment.range.end_line == 12
    assert char_comment.range.end_character == 19

@respx.mock
def test_review_mixed_inline_and_labels(runner):
    # Mock API
    route = respx.post("https://gerrit.example.com/a/changes/12345/revisions/current/review").mock(
        return_value=Response(200, json={"labels": {"Code-Review": 2}})
    )

    result = runner.invoke(
        cli,
        [
            "review",
            "12345",
            "--code-review",
            "+2",
            "--inline-comment",
            "src/main.py#10",
            "Nice fix",
        ],
        env={
            "GERRIT_URL": "https://gerrit.example.com",
            "GERRIT_USERNAME": "user",
            "GERRIT_PASSWORD": "password",
        },
    )

    assert result.exit_code == 0
    assert "Review sent to Change 12345" in result.output
    
    # Verify request payload
    assert route.called
    request_data = ReviewInput.model_validate_json(route.calls.last.request.content)
    
    assert request_data.labels is not None
    assert request_data.labels["Code-Review"] == 2
    
    assert request_data.comments is not None
    assert "src/main.py" in request_data.comments
    assert request_data.comments["src/main.py"][0].message == "Nice fix"
